/* file example.c66 */

/* this file shows the basic use of the software in an example */

/*

Description
-----------
The Real-time clock (RTC) is initialized with an generic date/time structure.
After running the software on the 80C166 target the interrupt-line should be High '1' 
for the 10 seconds (Do not forget a pull-up resistor on the interrupt-line).
(A not initialized RTC toggles it interrupt-line every half second).

After setting of the date/time the RTC is read at maximum speed. You should see 
in the following on your screen the RTC counting the time up every second (endless).

After 10 seconds the interrupt-line is activated and remains stable at Low '0'.

To stop the output RESET the target. If the RTC should also RESET you must power 
down the power supply of the RTC explicity for five seconds or so.


Programming
-----------
How "the first time" is evaluated:
For simplicity the RTC is always set to a known date every time the program is started.

You can evaluate the full example on a ready-compiled hex file (file EXAMPLE.H86).
Compiling the example should result in the same binary code (see file EXAMPLE.BAT).
The example MUST BE loaded under MONITOR control because it has no code to initialize 
the serial interface. Running the example from an EPROM do not work!

*/


/****************************************************************************/
/* start of code example                                                    */
/* for simplicity no error checking is done                                 */
/****************************************************************************/



#include "RTC.H"
#include <stdio.h>		/* for printf-function */
#include <reg166.h>



#define  CLRSCR   26



/****************************************************************************/
/*                                                                          */
/* function prototypes                                                      */
/*                                                                          */
/****************************************************************************/

void PrintTime(REAL_TIME Time);
void AlarmTime(REAL_TIME Time);



/****************************************************************************/
/*                                                                          */
/* function definitions                                                     */
/*                                                                          */
/****************************************************************************/

void PrintAlarm(REAL_TIME Time) {
   printf("   ");
   printf("%02X.%02X.,", Time.Date, Time.Month);
   printf(" %02X:%02X:%02X", Time.Hour, Time.Minute, Time.Second);
   printf("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b");
}

/****************************************************************************/

void PrintTime(REAL_TIME Time) {



   printf("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b");

   switch (Time.Weekday) {
      case SUNDAY:
         printf("SUN,");
         break;

      case MONDAY:
         printf("MON,");
         break;

      case TUESDAY:
         printf("TUE,");
         break;

      case WEDNESDAY:
         printf("WED,");
         break;

      case THURSDAY:
         printf("THU,");
         break;

      case FRIDAY:
         printf("FRI,");
         break;

      case SATURDAY:
         printf("SAT,");
         break;

      default:
         break;
   }

   printf(" %02X.%02X.%04X,", Time.Date, Time.Month, Time.Year);
   printf(" %02X:%02X:%02X", Time.Hour, Time.Minute, Time.Second);
}

/****************************************************************************/

void main(void) {
   BYTE              ClkID;            /* I2C-address of RTC                */
   REAL_TIME         Time;             /* structure for time handling       */
   BYTE              Alarm;



/* init serial port */
   P3  |= 0x0400;        /* SET PORT 3.10 OUTPUT LATCH (TXD)              */
   DP3 |= 0x0400;        /* SET PORT 3.10 DIRECTION CONTROL (TXD OUTPUT)  */
   DP3 &= 0xF7FF;        /* RESET PORT 3.11 DIRECTION CONTROL (RXD INPUT) */
   S0TIC = 0x80;         /* SET TRANSMIT INTERRUPT FLAG                   */
   S0RIC = 0x00;         /* DELETE RECEIVE INTERRUPT FLAG                 */
   S0BG  = 0x40;         /* SET BAUDRATE TO 9600 BAUD                     */
   S0CON = 0x8011;       /* SET SERIAL MODE                               */

   ClkID = 0xA2;                       /* base I2C-Address of RTC           */

   printf("%c", CLRSCR);
   printf("This is an example for the use of the PHYTEC RTC-Driver\n");

   Time.Weekday   = THURSDAY;          /* preset real time                  */
   Time.Date      = INT2BCD(1);
   Time.Month     = INT2BCD(1);
   Time.Year      = INT2BCD(1995);
   Time.Hour      = INT2BCD(0);
   Time.Minute    = INT2BCD(0);
   Time.Second    = INT2BCD(0);

   if (RTCSetTime(&Time, ClkID) != RTC_SUCCESSFUL) { /* write time to clock */
      printf("\n\n*** Unable to set time\7");
      printf("\nSystem halted.");
      while (1);
   }

   printf("The system time is now set to 00:00:00\n");

   printf("An alarm will occur after each 15 seconds which will be signaled by an beep\n\n");

   printf("\nRealtime                    Alarmtime\n");
   printf("                         ");
   PrintTime(Time);

   Time.Second += 0x15;

   if (RTCSetAlarm(&Time, ClkID) != RTC_SUCCESSFUL) { /* write time to clock */
      printf("\n\n*** Unable to set alarm\7");
      printf("\nSystem halted.");
      while (1);
   }

   PrintAlarm(Time);

   while (1) {

      if (RTCGetTime(&Time, ClkID) != RTC_SUCCESSFUL) { /* get time         */
         printf("\n\n*** Unable to get time\7");
         printf("\nSystem halted.");
         while (1);
      }

      PrintTime(Time);

      if (RTCGetAlarmStatus(&Alarm, ClkID) != RTC_SUCCESSFUL) {
         printf("\n\n*** Unable to get alarm status\7");
         printf("\nSystem halted.");
         while (1);
      }

      if (Alarm) {
         printf("\7");

         if (RTCGetAlarm(&Time, ClkID) != RTC_SUCCESSFUL) { /* get alarm    */
            printf("\n\n*** Unable to get alarm\7");
            printf("\nSystem halted.");
            while (1);
         }

         Time.Second += 0x15;          /* next alarm after 30 seconds       */

         if ((Time.Second & 0x0F) == 0x0A) {
            Time.Second &= 0xF0;
            Time.Second += 0x10;
         }

         if (Time.Second == 0x60) {    /* calc new alarm time               */
            Time.Second = 0x00;
            Time.Minute += 0x01;

            if ((Time.Minute & 0x0F) == 0x0A) {
               Time.Minute &= 0xF0;
               Time.Minute += 0x10;
            }

            if (Time.Minute == 0x60) {
               Time.Minute = 0x00;
               Time.Hour += 0x01;

               if ((Time.Hour & 0x0F) == 0x0A) {
                  Time.Hour &= 0xF0;
                  Time.Hour += 0x10;
               }

               if (Time.Hour == 0x24)
                  Time.Hour = 0x00;
            }
         }

         PrintAlarm(Time);

         if (RTCSetAlarm(&Time, ClkID) != RTC_SUCCESSFUL) {
                                       /* write alarm time to clock         */
            printf("\n\n*** Unable to set alarm\7");
            printf("\nSystem halted.");
            while (1);
         }
      }
   }
}

/****************************************************************************/
/* end of code example                                                      */
/****************************************************************************/
/* end example.c66                                                          */
/****************************************************************************/
